@@ -22,6 +22,10 @@
 #include <boost/spirit/home/support/iterators/ostream_iterator.hpp>
 #include <boost/spirit/home/support/unused.hpp>
 
+#if defined(BOOST_MSVC) && defined(BOOST_SPIRIT_UNICODE)
+#include <boost/spirit/home/support/char_encoding/unicode.hpp>
+#endif
+
 namespace boost { namespace spirit { namespace karma { namespace detail 
 {
     ///////////////////////////////////////////////////////////////////////////
@@ -64,7 +68,7 @@ namespace boost { namespace spirit { namespace karma { namespace detail
 
         template <typename T>
         void output(T const& value) 
-        { 
+        {
             // track position in the output 
             track_position_data.output(value);
         }
@@ -75,6 +79,18 @@ namespace boost { namespace spirit { namespace karma { namespace detail
             return track_position_data.get_count();
         }
 
+	// return the current line in the output
+	std::size_t get_line() const
+	{
+	    return track_position_data.get_line();
+	}
+
+	// return the current column in the output
+	std::size_t get_column() const
+	{
+	    return track_position_data.get_column();
+	}
+
     private:
         position_sink track_position_data;            // for position tracking
     };
@@ -110,7 +126,7 @@ namespace boost { namespace spirit { namespace karma { namespace detail
         }
 
         void output() 
-        { 
+        {
             ++count; 
         }
         std::size_t get_count() const { return count; }
@@ -147,7 +163,7 @@ namespace boost { namespace spirit { namespace karma { namespace detail
 
         template <typename T>
         void output(T const&) 
-        { 
+        {
             // count characters, if appropriate
             if (NULL != count)
                 count->output();
@@ -172,24 +188,33 @@ namespace boost { namespace spirit { namespace karma { namespace detail
     ///////////////////////////////////////////////////////////////////////////
     class buffer_sink : boost::noncopyable
     {
+       // wchar_t is only 16-bits on Windows. If BOOST_SPIRIT_UNICODE is
+       // defined, the character type is 32-bits wide so we need to make
+       // sure the buffer is at least that wide.
+#if (defined(_WIN32) || defined(__CYGWIN__)) && defined(BOOST_SPIRIT_UNICODE)
+       typedef spirit::char_encoding::unicode::char_type buffer_char_type;
+#else
+       typedef wchar_t buffer_char_type;
+#endif
+
     public:
         buffer_sink()
           : width(0) {}
 
         ~buffer_sink() 
-        { 
+        {
             tidy(); 
         }
 
         void enable(std::size_t width_) 
-        { 
+        {
             tidy();             // release existing buffer
             width = (width_ == std::size_t(-1)) ? 0 : width_;
             buffer.reserve(width); 
         }
 
         void tidy() 
-        { 
+        {
             buffer.clear(); 
             width = 0; 
         }
@@ -197,18 +222,18 @@ namespace boost { namespace spirit { namespace karma { namespace detail
         template <typename T>
         void output(T const& value)
         {
-            BOOST_STATIC_ASSERT(sizeof(T) <= sizeof(wchar_t));
+            BOOST_STATIC_ASSERT(sizeof(T) <= sizeof(buffer_char_type));
             buffer.push_back(value);
         }
 
         template <typename OutputIterator_>
         bool copy(OutputIterator_& sink, std::size_t maxwidth) const 
-        { 
+        {
 #if defined(BOOST_MSVC)
 #pragma warning(push)
 #pragma warning(disable: 4267)
 #endif
-            typename std::basic_string<wchar_t>::const_iterator end = 
+            typename std::basic_string<buffer_char_type>::const_iterator end = 
                 buffer.begin() + (std::min)(buffer.size(), maxwidth);
 
 #if defined(BOOST_MSVC)
@@ -219,12 +244,12 @@ namespace boost { namespace spirit { namespace karma { namespace detail
         }
         template <typename RestIterator>
         bool copy_rest(RestIterator& sink, std::size_t start_at) const 
-        { 
+        {
 #if defined(BOOST_MSVC)
 #pragma warning(push)
 #pragma warning(disable: 4267)
 #endif
-            typename std::basic_string<wchar_t>::const_iterator begin = 
+            typename std::basic_string<buffer_char_type>::const_iterator begin = 
                 buffer.begin() + (std::min)(buffer.size(), start_at);
 
 #if defined(BOOST_MSVC)
@@ -235,13 +260,13 @@ namespace boost { namespace spirit { namespace karma { namespace detail
         }
 
         std::size_t buffer_size() const 
-        { 
+        {
             return buffer.size();
         }
 
     private:
         std::size_t width;
-        std::basic_string<wchar_t> buffer;
+        std::basic_string<buffer_char_type> buffer;
     };
 
     ///////////////////////////////////////////////////////////////////////////
@@ -279,7 +304,7 @@ namespace boost { namespace spirit { namespace karma { namespace detail
     struct no_buffering_policy
     {
         no_buffering_policy() {}
-        no_buffering_policy(no_counting_policy const&) {}
+        no_buffering_policy(no_buffering_policy const&) {}
 
         template <typename T>
         bool output(T const& /*value*/) 
@@ -452,6 +477,9 @@ namespace boost { namespace spirit { namespace karma { namespace detail
         // plain output iterators are considered to be good all the time
         bool good() const { return true; }
 
+        // allow to access underlying output iterator
+        OutputIterator& base() { return *sink; }
+
     protected:
         // this is the wrapped user supplied output iterator
         OutputIterator* sink;
